/* ************************************************************************** */
/*                                                                            */
/*                                                        :::      ::::::::   */
/*   xpm.c                                              :+:      :+:    :+:   */
/*                                                    +:+ +:+         +:+     */
/*   By: cchauvet <cchauvet@student.42angouleme.fr  +#+  +:+       +#+        */
/*                                                +#+#+#+#+#+   +#+           */
/*   Created: 2023/01/04 13:53:03 by cchauvet          #+#    #+#             */
/*   Updated: 2023/01/06 18:46:43 by cchauvet         ###   ########.fr       */
/*                                                                            */
/* ************************************************************************** */

#include "solong.h"

static char	*name_generator(t_shape shape)
{
	char	*out;
	char	*size;
	char	*temp;

	size = ft_itoa(shape.size);
	if (size == NULL)
		return (NULL);
	temp = ft_strmerger(5, XPM_PATH, shape.shape, "_", shape.color, "_");
	if (temp == NULL)
	{
		free(size);
		return (NULL);
	}
	out = ft_strmerger(5, temp, size, "x", size, ".xpm");
	free(size);
	free(temp);
	return (out);
}

static char	*ft_map_gen(t_shape shape)
{
	if (ft_strcmp(shape.shape, "square") == 0)
		return (ft_square('a', shape.size, shape.size));
	if (ft_strcmp(shape.shape, "triangle") == 0)
		return (ft_triangle('a', 'b', shape.size, shape.size));
	return (NULL);
}

char	*ft_gen_xpm_content(t_shape shape)
{
	char	*content;
	char	*temp;
	char	*map;

	temp = ft_itoa(shape.size);
	if (temp == NULL)
		return (NULL);
	content = ft_strmerger(5, XPM_HEADER, temp, " ", temp, " 2 1\",\n\"");
	free(temp);
	if (content == NULL)
		return (NULL);
	temp = ft_strmerger(5, "/* XPM */\n", content, "a c ", shape.color, "\",\n\"");
	if (temp == NULL)
		return (NULL);
	free(content);
	map = ft_map_gen(shape);
	if (map == NULL)
		free(temp);
	if (map == NULL)
		return (NULL);
	content = ft_strmerger(6, temp, "b c ", shape.bcolor, "\",\n", map, "};");
	free(temp);
	free(map);
	return (content);
}

char	*ft_xpm_gen_file(t_shape shape)
{
	char	*file_content;
	char	*path;
	int		fd;

	path = name_generator(shape);
	if (path == NULL)
		return (NULL);
	fd = open(path, O_RDONLY);
	if (fd == -1)
	{
		file_content = ft_gen_xpm_content(shape);
		if (file_content == NULL)
		{
			free(path);
			return (NULL);
		}
		fd = open(path, O_WRONLY | O_CREAT, 0644);
		if (fd == -1)
		{
			free(path);
			free(file_content);
			return (NULL);
		}
		write(fd, file_content, ft_strlen(file_content));
		close(fd);
		free(file_content);
	}
	else
		close(fd);
	return (path);
}
